import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:mapbox_maps_flutter/mapbox_maps_flutter.dart';
import 'package:http/http.dart' as http;

void main() {
  runApp(const MaterialApp(
      debugShowCheckedModeBanner: false, home: AutocompleteMap()));
}

class AutocompleteMap extends StatefulWidget {
  const AutocompleteMap({super.key});

  @override
  State createState() => FullMapState();
}

class Debouncer {
  final Duration delay;
  Timer? _timer;

  Debouncer({required this.delay}){
    print("Debouncer constructor");
  }

  void debounce(Function() action) {
    _timer?.cancel();
    _timer = Timer(delay, action);
  }
}

class FullMapState extends State<AutocompleteMap> {
  MapboxMap? mapboxMap;
  CircleAnnotationManager? _circleAnnotationManager;
  double? lngCam;
  double? latCam;
  List<dynamic> coor = [];

  _onMapCreated(MapboxMap mapboxMap) async {
    this.mapboxMap = mapboxMap;
    this.mapboxMap?.getCameraState().then((value) {
      setState(() {
        coor = value.center['coordinates'] as List<dynamic>;
        lngCam = coor[0];
        latCam = coor[1];
      });
    });
  }

  final TextEditingController _searchController = TextEditingController();
  Debouncer debouncer = Debouncer(delay: const Duration(milliseconds: 100));
  String mainText = "";
  String secondText = "";
  List<dynamic> places = [];
  var details = {};
  bool isShow = false;
  bool isHidden = true;

  Future<void> fetchData(String input) async {
    try {
      // final url = Uri.parse(
      //     'https://rsapi.goong.io/Place/AutoComplete?api_key=qsy0OS8PcbxbmdNzOn8Gy0mSEdg4trKTgtcUD5DN&input=$input');
      final url = Uri.parse(
          'http://40.90.170.197:9999/place/autocomplete?input=$input&location=$latCam,$lngCam&limit=10');
      print('url $url');
      var response = await http.get(url);
      setState(() {
        final jsonResponse = jsonDecode(response.body);
        places = jsonResponse['predictions'] as List<dynamic>;
        print('url $url, size: ${places.length}');
        _circleAnnotationManager?.deleteAll();
        isShow = true;
        isHidden = true;
      });
    } catch (e) {
      // ignore: avoid_print
      print('$e');
    }
  }

  Widget _buildListView() {
    return ListView.separated(
      itemCount: places.length < 5 ? places.length : 5,
      shrinkWrap: true,
      separatorBuilder: (BuildContext context, int index) => const Divider(height: 1),
      itemBuilder: (context, index) {
        final coordinate = places[index];

        return ListTile(
          horizontalTitleGap: 5,
          title: Row(
            children: [
              const Icon(
                Icons.location_on_outlined,
                color: Colors.blue,
                size: 20,
              ),
              const SizedBox(width: 8,),
              SizedBox(
                width: 320,
                child: Text(
                  coordinate['description'],
                  softWrap: true,
                  style: const TextStyle(
                    color: Colors.black54,
                  ),
                ),
              )
            ],
          ),
          onTap: () async {
            setState(() {
              isShow = false;
              isHidden = false;
            });

            final url = Uri.parse(
                'http://40.90.170.197:9999/place/detail?place_id=${coordinate['place_id']}');
            var response = await http.get(url);
            final jsonResponse = jsonDecode(response.body);
            details = jsonResponse['result'];

            // ignore: no_leading_underscores_for_local_identifiers
            mapboxMap?.setCamera(CameraOptions(
                center: Point(
                        coordinates: Position(
                            details['geometry']['location']['lng'],
                            details['geometry']['location']['lat']))
                    .toJson(),
                zoom: 12.0));

            mapboxMap?.flyTo(
                CameraOptions(
                    anchor: ScreenCoordinate(x: 0, y: 0),
                    zoom: 15,
                    bearing: 0,
                    pitch: 0),
                MapAnimationOptions(duration: 2000, startDelay: 0));
            mapboxMap?.annotations
                .createCircleAnnotationManager()
                .then((value) async {
              setState(() {
                _circleAnnotationManager =
                    value; // Store the reference to the circle annotation manager
              });

              value.create(
                CircleAnnotationOptions(
                  geometry: Point(
                          coordinates: Position(
                              details['geometry']['location']['lng'],
                              details['geometry']['location']['lat']))
                      .toJson(),
                  circleColor: Colors.blue.value,
                  circleRadius: 12.0,
                ),
              );
            });
            _searchController.text = coordinate['description'];
            mainText = coordinate['structured_formatting']['main_text'];
            secondText = coordinate['structured_formatting']['secondary_text'];
          },
        );
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: AppBar(
          title: const Text('Autocomplete'),
        ),
        body: Stack(children: [
          SizedBox(
            child: MapWidget(
              key: const ValueKey("mapWidget"),
              resourceOptions: ResourceOptions(
                  accessToken:
                      "pk.eyJ1IjoibG9uZ25naGllbSIsImEiOiJjbGhyMnhwdGEyOXNmM3FzMXA5Z3U1c2VsIn0.6fGbge-wVwxzhgKFAu8pkg"),
              cameraOptions: CameraOptions(
                  center: Point(coordinates: Position(105.83991, 21.02800))
                      .toJson(),
                  zoom: 12.0),
              styleUri: MapboxStyles.DARK,
              textureView: true,
              onMapCreated: _onMapCreated,
            ),
          ),
          Container(
              height: 70,
              alignment: Alignment.topLeft,
              margin: const EdgeInsets.fromLTRB(10, 10, 10, 0),
              padding: const EdgeInsets.fromLTRB(10, 0, 10, 0),
              decoration: BoxDecoration(color: Colors.grey[200]),
              child: Row(
                children: [
                  Expanded(
                    child: Column(
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        Container(
                            padding: const EdgeInsets.only(left: 4),
                            decoration:
                                const BoxDecoration(color: Colors.white),
                            child: Row(
                              children: [
                                const Icon(
                                  Icons.location_on_outlined,
                                  color: Colors.blue,
                                  size: 20,
                                ),
                                Expanded(
                                    child: Padding(
                                  padding:
                                      const EdgeInsets.only(left: 4, right: 8),
                                  child: TextField(
                                    controller: _searchController,
                                    onChanged: (String text) {
                                      print("onChanged: $text");
                                      debouncer.debounce(() => fetchData(text));
                                      isHidden = true;
                                    },
                                    decoration: const InputDecoration(
                                        hintText: "Nhập địa điểm",
                                        border: InputBorder.none,
                                        hintStyle: TextStyle(
                                            color: Colors.black54,
                                            fontSize: 16)),
                                  ),
                                )),
                              ],
                            )),
                      ],
                    ),
                  ),
                ],
              )),
          if (isShow == true)
            Container(
              margin: const EdgeInsets.fromLTRB(10, 85, 10, 0),
              padding: const EdgeInsets.fromLTRB(0, 0, 0, 0),
              decoration: const BoxDecoration(color: Colors.white),
              child: _buildListView(),
            ),
          isHidden
              ? const Card()
              : Align(
                  alignment: Alignment.bottomCenter,
                  child: Container(
                      height: 120,
                      margin: const EdgeInsets.fromLTRB(0, 200, 0, 0),
                      padding: const EdgeInsets.fromLTRB(0, 10, 0, 10),
                      alignment: Alignment.topLeft,
                      decoration: const BoxDecoration(
                          color: Colors.white,
                          borderRadius: BorderRadius.only(
                              topLeft: Radius.circular(12),
                              topRight: Radius.circular(12))),
                      child: Padding(
                        padding:
                            const EdgeInsets.only(left: 16, right: 16, top: 10),
                        child: ListView(
                          children: [
                            Text(
                              mainText,
                              style: const TextStyle(
                                  color: Colors.black, fontSize: 18),
                            ),
                            const Padding(padding: EdgeInsets.only(top: 12)),
                            Text(
                              secondText,
                              style: const TextStyle(
                                  color: Colors.black54, fontSize: 15),
                            )
                          ],
                        ),
                      )),
                ),
        ]));
  }
}
